/***************************************************************
	SPI Module
	
 Il modulo seguente interfaccia l'ADC della DE0Nano generando
 un clock tramite un contatore (e non un PLL).
 I dati sono letti dall'ADC solo dal canale 1
 
 PROBLEMATICHE RISCONTRATE:
 --------------------------
 Premesso che non riesco a simulare in GATE-Level, ma solo in RTL, sembra comunque che ci possa essere
 un problema perchè non ho trovato un modo semplice di bloccare sclk_count.
 Infatti se arriva un nuovo impulso di abilitazione (e quindi "ena" torna a 1), il clock comincia ad
 uscire al valore che ha in quel momento, mentre sarebbe bene che cominciasse sempre da 1.
 ***************************************************************/

module SPI
(	input		clk,				// clock a 50MHz
	input		enable,			// posedge -> comincia la trasmissione
	input		DOUT,				// dati in uscita dall'ADC
	output	DIN,				// dati in ingresso all'ADC
	output	CS,				// chip select, "1" a riposo
	output	SCLK,				// clock dell'SPI
	output	[15:0] DATA);	// dati effettivamente ricevuti

reg 	[4:0] sclk_count = 5'd0;	// 4 bit per la riscalature e il clock è il 5° bit
reg	[4:0] index = 5'd0;	// il 4° bit è l'enable
reg	ena = 1'b0;

reg	[15:0] dout = 16'd0;

wire	reset_index;

// Riscalatura del clock: il risultato deve essere tra 0.8 e 3.2 MHz
// con 4 bit la frequenza massima sarà di 3.125MHz
always @(posedge clk)
begin
//	if(!ena)
//		sclk_count <= {5{1'b1}}; 	//5'hFF; deve rimanere a "1" quando è tutto spento
//	else
		sclk_count <= sclk_count + 1'b1;		
end

// Abilita la trasmissione
always @(posedge clk)
begin
	if (index[4])
		ena = 1'b0;
	else
	if (enable)
		ena = 1'b1;
end

// campionamento dei dati sul fronte positivo
always @(posedge sclk_count[4])	
begin
	if(ena) 				// ena viene messo a 0 quando si ha l'overflow dell'index
	begin
		dout[index] = DOUT;
		index = index + 1'b1;
	end
	else
	if(!enable && index[4])
		index = 5'd0;
end

assign DIN = 1'b0;	// Trasmissione basata sul nuovo clock: devo trasmettere solo l'address che è 0
assign CS = !ena;
assign SCLK = sclk_count[4] || !ena;
assign DATA = (!ena) ? dout : DATA;

endmodule
