/*
	PWM FADE
	Date:		15/09/2014
	Target:	Cyclone IV
	Board:	DE0-NANO
	
 OBIETTIVO:
 ----------
 Realizzare l'accensione dei leg in PWM simulando lo scorrere di un cursore.
 Basato su DE0Nano_2level.

 NOTE IMPLEMENTATIVE:
 --------------------
 - si applica un duty variabile sui vari led
 - Per avere a 1ms 2^32 occorre uno step di 171798@25MHz, 69184@50MHz
 - Sin e Cos sono memorizzati a 15 bit in modo che poi il segno venga gestito successivamente
 - La creazione dell'angolo viene fatta con due banchi di RAM, mentre sarebbe meglio creare
  una pipe per calcolare l'angolo del seno e del coseno e otenere i due valori.
 - uso il GENERATE LOOP per istanziare gli 8 led per fare prima. Nota: il begin deve avere
  obbligatoriamente un nome.
 
 NOTA IMPORTANTISSIMA DI TEMPORIZZAZIONE:
 ----------------------------------------
 - Per risolvere il problema del passaggio per lo 0 occorre sistemare le temporizzazioni.
  Es: se clk=25 e clk_=12.5 si mette: Theta a 25M, Sin e Cos a 25M e il cambio segno a 12.5M.
  In altre parole il cambio segno deve essere più lento del resto
  Se invece è Theta e cambio segno a 25M e SinCos a 12.5 o 50M non va perchè nell'istante in
  cui theta cambia uno degli ultimi 2 bit, il sistema si aggiorna temporaneamente.
 - La CARRIER, siccome esegue molti calcoli DEVE essere un registro e non un WIREm anche se
  non capisco perchè.

 RISULTATO:
 ----------
 Funziona.
 E' stato verificato anche il comportamento con l'oscilloscopio: funziona e si vede anche il
 ritardo di 5us (2.5 se il clock è 50MHz).
*/

module main(
	input		clk,								// clock da 25Mz, quindi a 50MHz viaggerà a 2kHz la fSW
	output	[7:0] LED						// comando positivo
);

reg signed	[31:0] dutyTmp [7:0];
reg signed	[31:0] duty [7:0];
reg [31:0]	cnt = 0;						// contatore di base da cui generare la carrier
reg slope = 1'b1;
reg signed [31:0] carrier;				// Non so perchè ma va gestito per forza come registro e non come wire

//wire signed [31:0] duty = 32'hC0000000;	// dovrebbe essere il 75%
wire sync;									// Equivalente di un comando di sync da un uC/DSP. Ora lo genero io.

integer i;

// NOTA: se metto NEGEDGE non funziona. Evidentemente siccome uso l'= i ritardi sono sufficienti
// per memorizzare il tutto. Ad ogni modo per pulizia ho portato questa assegnazione prima del contatore.
//---------------------

initial
begin
	for (i=0; i<8; i = i+1)
	begin
		dutyTmp[i] <= 32'd536870912 * i;  //32'd268435456 * i;	// 32'd536870912 * i;
		duty[i] <= 0;
	end
end

// Generazione dell'angolo: più il clock è lento, maggiore è la risoluzione numerica
// dello step, ma inferiore quella temporale e quindi in tensione 
always @(posedge clk)
begin	: Counter
	slope = cnt[31];						// lo metto qui con l'"=" per essere certo di passare un ciclo. Con negedge non funziona.
	cnt = cnt + 32'd171799;				// step: 171799 = 1kHz @ clk=25M; cambiando questo si cambia la frequenza di switching
	carrier = (((cnt ^ {32{cnt[31]}}) - {30{1'b1}}) <<< 1'd1);	// NOTA: si perde comunque il bit meno significativo!
end

assign sync = slope ^ cnt[31];

// Generazione del DutyCyle
always @(posedge clk)
begin
	for (i=0; i<8; i = i+1)
	begin
		dutyTmp[i] <= dutyTmp[i] + 32'd54;
		duty[i] = (((dutyTmp[i] ^ {32{dutyTmp[i][31]}}) - {30{1'b1}}) <<< 1'd1);
	end
end

// Generazione di tutte le 8 PWM
genvar j;
generate
for (j = 0; j < 8; j = j + 1) begin : pwmInstance
	PWM	pwmX
	(	.clk(clk),
		.syncEnable(sync),
		.mod($signed(duty[j])),
		.carrier($signed(carrier)),
		.P(LED[j]));
end endgenerate

/**
PWM	pwm1
(	.clk(clk),
	.syncEnable(sync),
	.mod($signed(duty[0])),
	.carrier($signed(carrier)),
	.P(LED[0]));

PWM	pwm2
(	.clk(clk),
	.syncEnable(sync),
	.mod($signed(duty[1])),
	.carrier($signed(carrier)),
	.P(LED[1]));

PWM	pwm3
(	.clk(clk),
	.syncEnable(sync),
	.mod($signed(duty[2])),
	.carrier($signed(carrier)),
	.P(LED[2]));

PWM	pwm4
(	.clk(clk),
	.syncEnable(sync),
	.mod($signed(duty[3])),
	.carrier($signed(carrier)),
	.P(LED[3]));
	
PWM	pwm5
(	.clk(clk),
	.syncEnable(sync),
	.mod($signed(duty[4])),
	.carrier($signed(carrier)),
	.P(LED[4]));

PWM	pwm6
(	.clk(clk),
	.syncEnable(sync),
	.mod($signed(duty[5])),
	.carrier($signed(carrier)),
	.P(LED[5]));

PWM	pwm7
(	.clk(clk),
	.syncEnable(sync),
	.mod($signed(duty[6])),
	.carrier($signed(carrier)),
	.P(LED[6]));

PWM	pwm8
(	.clk(clk),
	.syncEnable(sync),
	.mod($signed(duty[7])),
	.carrier($signed(carrier)),
	.P(LED[7]));
**/
endmodule
