#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    MySocket = new QTcpSocket(this);
    connect(MySocket, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(displayError(QAbstractSocket::SocketError)));
    connect(MySocket, SIGNAL(readyRead()), this, SLOT(readData()));
}

MainWindow::~MainWindow()
{
    if(MySocket->isOpen())
        MySocket->close();

    delete MySocket;

    delete ui;
}

// readData()  la funzione pi importante perch completa il processo di lettura
// o scrittura in funzione della funzione attivata.
void MainWindow::readData()
{
    int size, i;
    char bufIn[1460];
    QString bufOut;

    // "size" va letto per forza. Mi chiedo comunque se sia necessario azzerare il buffer.
    size = MySocket->bytesAvailable();
    if(size > 1460)
        size = 1460;

//    for(i=0; i<size; i++)
//        bufIn[i] = '\0';

    MySocket->read(bufIn,size);
    bufIn[size-1] = '\0';

    // Plotto i dati provenienti dal TDO
    ui->textEdit->append(QString("TDO: ")+QString(bufIn));
}

void MainWindow::on_ConnectButton_clicked()
{
    if(MySocket->isOpen())
    {
        MySocket->disconnectFromHost();     // non ricordo perch, ma mi pare si meglio (see doc on line)
        MySocket->close();
        ui->ConnectButton->setText("Connect");
        ui->textEdit->append("Client disconnected from localhost.");
    }
    else
    {
        MySocket->abort();                  // per precauzione, l'ho visto in un esempio
        MySocket->connectToHost("localhost", JTAG_PORT);
        if(MySocket->waitForConnected(2000))
        {
            ui->ConnectButton->setText("Disconnect");
            ui->textEdit->append("Connected to localhost on port " + QString("%1").arg(MySocket->localPort()));
        }
        else
        {
            ui->ConnectButton->setText("Connect");
            ui->textEdit->append("Error while connecting...");
            MySocket->close();
        }
    }
}

void MainWindow::on_SendButton_clicked()
{
    QString bufOut;

    if(!MySocket->isOpen())
    {
        ui->textEdit->append("Client is not opened!");
        return;
    }

    bufOut.sprintf("%s\n",ui->CommandLine->text().toStdString().c_str());
    MySocket->write(bufOut.toStdString().c_str(), bufOut.length());
//    bufOut.sprintf("0x%X\n",1001);
//    MySocket->write(bufOut.toStdString().c_str(), bufOut.length());
    ui->textEdit->append(QString("Send: ") + ui->CommandLine->text() + QString("\tlen: ")+QString::number(bufOut.toStdString().length()));

}
